from six import iteritems

def place_files_in_operations(operations, files_map, files):
    path_to_key_iter = (
        (value.split('.'), key)
        for (key, values) in iteritems(files_map)
        for value in values
    )
    # Since add_files_to_operations returns a new dict/list, first define
    # output to be operations itself
    output = operations
    for path, key in path_to_key_iter:
        file_obj = files[key]
        output = add_file_to_operations(output, file_obj, path)
    return output


def add_file_to_operations(operations, file_obj, path):
    if not path:
        if operations is not None:
            raise ValueError('Path in map does not lead to a null value')
        return file_obj
    if isinstance(operations, dict):
        key = path[0]
        sub_dict = add_file_to_operations(operations[key], file_obj, path[1:])
        return new_merged_dict(operations, {key: sub_dict})
    if isinstance(operations, list):
        index = int(path[0])
        sub_item = add_file_to_operations(operations[index], file_obj, path[1:])
        return new_list_with_replaced_item(operations, index, sub_item)
    raise TypeError('Operations must be a dict or a list of dicts')


def new_merged_dict(*dicts):
    # Necessary for python2 support
    output = {}
    for d in dicts:
        output.update(d)
    return output


def new_list_with_replaced_item(input_list, index, new_value):
    # Necessary for python2 support
    output = [i for i in input_list]
    output[index] = new_value
    return output



from flask import request
from flask_graphql import GraphQLView
from graphql_server import load_json_body

from ..utils import place_files_in_operations


class FileUploadGraphQLView(GraphQLView):

    def parse_body(self):
        content_type = request.mimetype
        if content_type == 'multipart/form-data':
            operations = load_json_body(request.form.get('operations', '{}'))
            files_map = load_json_body(request.form.get('map', '{}'))
            return place_files_in_operations(
                operations,
                files_map,
                request.files
            )
        return super(FileUploadGraphQLView, self).parse_body()



import graphene

class Upload(graphene.types.Scalar):
    @staticmethod
    def serialize(value):
        return value

    @staticmethod
    def parse_literal(node):
        return node

    @staticmethod
    def parse_value(value):
        return value



# Always prefer setuptools over distutils
from setuptools import setup
# To use a consistent encoding
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
# try:
    # import pypandoc
    # long_description = pypandoc.convert('README.md', 'rst')
# except(IOError, ImportError):
#     long_description = open('README.md').read()

long_description = open('README.rst').read()

flask_requires = [
    'Flask>=1.0.2',
    'graphene>=2.1.2',
    'Flask-Graphql>=2.0.0',
]

django_requires = []
tests_require = flask_requires + django_requires + [
    'coverage>=4.5.1',
    'pytest>=3.7.4',
    'pytest-cov>=2.6.0',
]

setup(
    name='graphene_file_upload',
    packages=[
        'graphene_file_upload', # this must be the same as the name above
        'graphene_file_upload.flask',
        'graphene_file_upload.django',
    ],
    version='1.2.0',
    description='Lib for adding file upload functionality to GraphQL mutations in Graphene Django and Flask-Graphql',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='Lucas McGartland',
    author_email='lucasmcgartland@gmail.com',
    url='https://github.com/lmcgartland/graphene-file-upload', # use the URL to the github repo
    # download_url = 'https://github.com/lmcgartland/graphene-file-upload/archive/0.1.0.tar.gz',
    keywords=['graphql', 'graphene', 'apollo',  'upload'], # arbitrary keywords
    classifiers=[],
    install_requires=[
        'six>=1.11.0',
    ],
    tests_require=tests_require,
    extras_require={
        'test': tests_require,
        'flask': flask_requires,
        'django': django_requires,
    },
)



import json
from graphene_django.views import GraphQLView

from .utils import place_files_in_operations


class FileUploadGraphQLView(GraphQLView):

    def parse_body(self, request):
        content_type = self.get_content_type(request)
        if content_type == 'multipart/form-data':
            operations = json.loads(request.POST.get('operations', '{}'))
            files_map = json.loads(request.POST.get('map', '{}'))
            return place_files_in_operations(
                operations,
                files_map,
                request.FILES
            )
        return super(FileUploadGraphQLView, self).parse_body(request)
